@@ -1,6 +1,6 @@
 # Run tests
 
-library(testthat)
+if(!require(testthat)) stop("`testthat` must be available to run tests.")
 library(diffobj)
 
 local({                                         # so we can use `on.exit`
@@ -32,6 +32,8 @@ local({                                         # so we can use `on.exit`
 
   on.exit(options(old.opts))
 
+  RNGversion("3.5.2");
+
   valgrind <- FALSE
   if(!valgrind) {
     test.res <- test_dir(
@@ -78,4 +80,5 @@ local({                                         # so we can use `on.exit`
   } else {
     source('valgrind/tests-valgrind.R')
   }
+  RNGversion(as.character(getRversion()))
 })
