@@ -6,6 +6,8 @@ if(!identical(basename(getwd()), "testthat"))
 
 rdsf <- function(x)
   file.path(getwd(), "helper", "context", sprintf("%s.rds", x))
+txtf <- function(x)
+  file.path(getwd(), "helper", "context", sprintf("%s.txt", x))
 
 test_that("interesting context values", {
   expect_equal_to_reference(
@@ -34,4 +36,25 @@ test_that("with line limit", {
     as.character(diffChr(chr.9, chr.10, context="auto", line.limit=25)),
     rdsf(500)
   )
+  # default to min context
+
+  a <- b <- letters
+  b[c(3, 20)] <- LETTERS[c(3,20)]
+  expect_known_output(
+    show(diffChr(a, b, line.limit=c(20, 10), context='auto', format='raw')),
+    txtf(100)
+  )
+  # trim hunks in auto-context mode
+
+  a <- b <- letters
+  b[c(3, 10, 20)] <- LETTERS[c(3,10,20)]
+  expect_known_output(
+    show(
+      diffChr(
+        a, b, hunk.limit=c(2, 1), context=auto_context(1, 5), line.limit=20,
+        format='raw'
+      )
+    ),
+    txtf(200)
+  )
 })
